@@ -3,28 +3,10 @@ import struct
 import datetime
 from typing import List
 
-from osrparse.enums import Mod, GameMode
+from osrparse.utils import (Mod, GameMode, ReplayEvent, ReplayEventOsu,
+    ReplayEventCatch, ReplayEventMania, ReplayEventTaiko)
 
-class ReplayEvent():
-    def __init__(self, time_since_previous_action: int, x: float, y: float, keys_pressed: int):
-        self.time_since_previous_action = time_since_previous_action
-        self.x = x
-        self.y = y
-        self.keys_pressed = keys_pressed
-
-    def __str__(self):
-        return f"{self.time_since_previous_action} ({self.x}, {self.y}) {self.keys_pressed}"
-
-    def __eq__(self, other):
-        if not isinstance(other, ReplayEvent):
-            return False
-        return (self.time_since_previous_action == other.time_since_previous_action
-            and self.x == other.x and self.y == other.y and self.keys_pressed == other.keys_pressed)
-
-    def __hash__(self):
-        return hash((self.time_since_previous_action, self.x, self.y, self.keys_pressed))
-
-class Replay():
+class Replay:
     # first version with rng seed value added as the last frame in the lzma data
     LAST_FRAME_SEED_VERSION = 20130319
     _BYTE = 1
@@ -53,6 +35,7 @@ class Replay():
         self.timestamp = None
         self.play_data = None
         self.replay_id = None
+        self.replay_length = None
         self._parse_replay_and_initialize_fields(replay_data, pure_lzma, decompressed_lzma)
 
     def _parse_replay_and_initialize_fields(self, replay_data, pure_lzma, decompressed_lzma):
@@ -134,19 +117,31 @@ class Replay():
 
     def _parse_play_data(self, replay_data):
         offset_end = self.offset+self.replay_length
-        if self.game_mode != GameMode.STD:
-            self.play_data = None
-        else:
-            datastring = lzma.decompress(replay_data[self.offset:offset_end], format=lzma.FORMAT_AUTO).decode('ascii')[:-1]
-            events = [eventstring.split('|') for eventstring in datastring.split(',')]
-            self.play_data = [ReplayEvent(int(event[0]), float(event[1]), float(event[2]), int(event[3])) for event in events]
+        datastring = lzma.decompress(replay_data[self.offset:offset_end], format=lzma.FORMAT_AUTO).decode('ascii')[:-1]
+        events = [eventstring.split('|') for eventstring in datastring.split(',')]
+
+        if self.game_mode is GameMode.STD:
+            self.play_data = [ReplayEventOsu(int(event[0]), float(event[1]), float(event[2]), int(event[3])) for event in events]
+        if self.game_mode is GameMode.TAIKO:
+            self.play_data = [ReplayEventTaiko(int(event[0]), float(event[1]), int(event[3])) for event in events]
+        if self.game_mode is GameMode.CTB:
+            self.play_data = [ReplayEventCatch(int(event[0]), float(event[1]), int(event[3])) for event in events]
+        if self.game_mode is GameMode.MANIA:
+            self.play_data = [ReplayEventMania(int(event[0]), int(event[1])) for event in events]
+
         self.offset = offset_end
 
         if self.game_version >= self.LAST_FRAME_SEED_VERSION and self.play_data:
-            if self.play_data[-1].time_since_previous_action != -12345:
-                print("The RNG seed value was expected in the last frame, but was not found. "
-                      f"\nGame Version: {self.game_version}, version threshold: "
-                      f"{self.LAST_FRAME_SEED_VERSION}, replay hash: {self.replay_hash}")
+            if self.play_data[-1].time_delta != -12345:
+                pass
+                # I've disabled this warning temporarily as it turns out that
+                # many replays (perhaps all replays in non-std gamemodes?) don't
+                # have an RNG seed value even after the expected version, so
+                # this was more of an annoying false positive than anything.
+
+                # print("The RNG seed value was expected in the last frame, but was not found. "
+                #       f"\nGame Version: {self.game_version}, version threshold: "
+                #       f"{self.LAST_FRAME_SEED_VERSION}, replay hash: {self.replay_hash}")
             else:
                 del self.play_data[-1]
 
@@ -158,9 +153,9 @@ class Replay():
         else:
             datastring = lzma.decompress(lzma_string, format=lzma.FORMAT_AUTO).decode('ascii')[:-1]
         events = [eventstring.split('|') for eventstring in datastring.split(',')]
-        self.play_data = [ReplayEvent(int(event[0]), float(event[1]), float(event[2]), int(event[3])) for event in events]
+        self.play_data = [ReplayEventOsu(int(event[0]), float(event[1]), float(event[2]), int(event[3])) for event in events]
 
-        if self.play_data[-1].time_since_previous_action == -12345:
+        if self.play_data[-1].time_delta == -12345:
             del self.play_data[-1]
 
     def _parse_replay_id(self, replay_data):
